\\ Copyright 2015, 2016, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
read("geom-inertia.gp");
read("geom-inertia.gp");   \\ safe to multiple reads
read("geom-inertia.gp");
read("test-helpers.gp");

option_verbose = 0;

sqrt3 = geom_sqrt_as_quad(3);  /* sqrt(3) */
check_equal(3, sqrt3^2);

sqrt3i = geom_sqrt_as_quad(-3);  /* sqrt(3)*I */
check_equal(-3, sqrt3i^2);

w3 = -1/2 + 1/2*sqrt3i;
check_equal(1, w3^3);


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_points()

{
  check_nearly(geom_inertia_of_points([6,4]),
               geom_inertia_of_point(6) + geom_inertia_of_point(4),
               "geom_inertia_of_points()");
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_path()

{
check_nearly(geom_inertia_of_path([1, 2, 2+I]),
             geom_inertia_of_segment(1, 2) + geom_inertia_of_segment(2, 2+I));
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_Iz()
\\ geom_inertia_Ixz()
\\ geom_inertia_Iyz()

{
  my(m = [1, 2, 3;
          2, 4, 5;
          3, 5, 6]);
  check_equal(geom_inertia_Ix(m), 1);
  check_equal(geom_inertia_Iy(m), 4);
  check_equal(geom_inertia_Iz(m), 6, "geom_inertia_Iz()");

  check_equal(geom_inertia_Ixy(m), -2);
  check_equal(geom_inertia_Ixz(m), -3);
  check_equal(geom_inertia_Iyz(m), -5);
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_triangle_xbase() -- unused, left to general triangle or poly

\\                          t = h*b/(b - a)
\\      |--->a
\\    h      *
\\    ^     /  \
\\    |    /     \
\\    |   /        \
\\    |  /           \
\\      *-------------+--
\\       ------------>b
\\
geom_inertia_of_triangle_xbase(b,h,a) =
{
  b*h*geom_inertia_of_Ix_Iy_Ixy(1/12*h^2,                 \\ Ix
                                1/12*(b^2 + a*b + a^2),   \\ Iy
                                1/24*b*h + 1/12*a*h);     \\ Ixy
}
{
addhelp(geom_inertia_of_triangle_xbase,
"M = geom_inertia_of_triangle_xbase(b, h, a)
Return an inertia matrix for a triangle with a corner at the origin, base
length b along the X axis, height h, and peak at x=a.  Should have b,h >=0,
but a can be positive or negative.

The triangle is taken to have mass equal to its area, uniformly distributed.
Multiply the result to change density if desired.");
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_rectangle_xbase()  -- unused
\\
\\ Think rectangle centred at the origin is more useful than corner at the
\\ origin, so this omitted for now.

\\      |
\\    h +---------*
\\      |         |
\\      |         |
\\      |         |
\\      0---------+---*
\\                w
\\
geom_inertia_of_rectangle_xbase(w,h) =
{
  geom_inertia_of_Ix_Iy_Ixy(1/3 * w * h^3,
                            1/3 * w^3 * h,
                            1/4*w^2*h^2);
}
{
addhelp(geom_inertia_of_rectangle_xbase,
"M = geom_inertia_of_rectangle_xbase(b, h, a)
Return the inertia matrix for a rectangle of width w and height h with a
corner at the origin and side along the X axis.  Should have have w,h >= 0.
The rectangle is taken to have mass equal to is area w*h, uniformly
distributed.");
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_principal_xydir()

check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([1,0])), \
            [0,1], "x axis point principal");
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([-1,0])), \
            [0,1], "x axis point principal");

\\ perpendicular to 3,4 is -4,3 == 4,-3
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([3,4])), \
            8*[4,-3]);
check_equal(geom_inertia_I_at_pdir(geom_inertia_of_point([3,4]), [4,-3]), 25);
check_equal(geom_inertia_I_at_pdir(geom_inertia_of_point([3,4]), [3,4]), 0);
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([-3,4])), \
            8*[4,3]);
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([3,-4])), \
            8*[4,3]);
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([-3,-4])), \
            8*[4,-3]);

\\ check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([3,4])), \
\\             6*[3,4]);
\\ check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([-3,4])), \
\\             6*[3,-4]);
\\ check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([3,-4])), \
\\             6*[3,-4]);
\\ check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([-3,-4])), \
\\             6*[3,4]);

{
  my(M=geom_inertia_of_point([3,4]) + 2*geom_inertia_of_point([-4,3]));
  \\ print(M);
  check_equal(geom_inertia_principal_2xydir(M), [-7,24]);
  check_equal(geom_inertia_principal_xydir(M), 4*[6,8]);
}

check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([-5,0])), \
            [0,25]);
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([5,0])), \
            [0,25]);
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([0,5])), \
            [50,0]);
check_equal(geom_inertia_principal_xydir(geom_inertia_of_point([0,5])), \
            [50,0]);

check_equal(geom_point_rotate_xydir(['x,'y],[0,1]), \
            [-'y,'x]);


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_segment()

{
  check_equal(geom_inertia_of_segment_m1(1,1),
              geom_inertia_of_point(1),
              "geom_inertia_of_segment_m1() as a single point");
}
{
  check_equal(geom_inertia_of_segment(-2,2),
              geom_inertia_of_segment_m1(-2,2) * 4,
              "geom_inertia_of_segment() cf m1");
}
{
  check_equal(geom_inertia_of_segment(-1,0) + geom_inertia_of_segment(0,1),
              geom_inertia_of_segment(-1,1),
              "geom_inertia_of_segment() in two parts");
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_path_m1()

\\ check_equal(geom_inertia_of_path_m1([]), \
\\             [0,0,0; 0,0,0; 0,0,0]);
check_equal(geom_inertia_of_path_m1([0]), \
            [0,0,0; 0,0,0; 0,0,0]);
check_equal(geom_inertia_of_path_m1([1+2*I]), \
            geom_inertia_of_point(1+2*I));
check_equal(geom_inertia_of_path_m1([1,I]), \
            geom_inertia_of_segment_m1(1,I));
check_equal(geom_inertia_of_path_m1([1,2,3,4]), \
            geom_inertia_of_segment_m1(1,4));


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_triangle()

check_equal(geom_inertia_of_triangle_m1(0,1,I), \
            [1/6, -1/12, 0; -1/12, 1/6, 0; 0, 0, 1/3]);
check_equal(geom_inertia_of_triangle_m1(0,I,1), \
            [1/6, -1/12, 0; -1/12, 1/6, 0; 0, 0, 1/3], \
            "clockwise");


{
  check_equal(geom_inertia_of_triangle_m1(1, w3, w3^2),
              [1/8, 0,0;  0, 1/8, 0;  0, 0, 1/4]);
  check_equal(geom_inertia_of_triangle(1, w3, w3^2),
              [3/32*sqrt3, 0,0;  0, 3/32*sqrt3, 0;  0, 0, 3/16*sqrt3]);
}

{
  my(w3v     = [-1/2; 1/2*sqrt3; 0],
     w3conjv = [-1/2; -1/2*sqrt3; 0]);
  check_equal(geom_inertia_of_triangle_m1([1;0;0], w3v, w3conjv),
              [1/8, 0,0;  0, 1/8, 0;  0, 0, 1/4]);

  my(z=[5;7;13]);
  my(m1=geom_inertia_of_triangle_m1([1;0;0], w3v, w3conjv)
        + geom_inertia_of_point(z));
  my(m2=geom_inertia_of_triangle_m1([1;0;0]+z, w3v+z, w3conjv+z));
  check_equal(m2,m1);
}

inertia_of_triangle_by_point_masses(p1,p2,p3) =
{
  my(m1=(p1+p2)/2,
     m2=(p2+p3)/2,
     m3=(p3+p1)/2);
    1/3 * geom_inertia_of_point(m1)
  + 1/3 * geom_inertia_of_point(m2)
  + 1/3 * geom_inertia_of_point(m3);
}
check_equal(geom_inertia_of_triangle_m1([x1,y1,z1],[x2,y2,z2],[x3,y3,z3]), \
            inertia_of_triangle_by_point_masses([x1,y1,z1],[x2,y2,z2],[x3,y3,z3]));

{
  check_equal(geom_inertia_of_triangle_m1(1,2,3)
              != geom_inertia_of_segment_m1(1,3),
              1,
              "triangle h=0 not equivalent to segment");
}
{
  check_equal(geom_inertia_of_triangle_m1(1,2,3) * 1/2
              + geom_inertia_of_triangle_m1(1,1,2) * 1/4
              + geom_inertia_of_triangle_m1(2,3,3) * 1/4,
              geom_inertia_of_segment_m1(1,3));
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_circle()

check_nearly(Pi*geom_inertia_of_circle_m1(1), \
             geom_inertia_of_circle(1), \
             "geom_inertia_of_circle() vs m1");
check_nearly(Pi*2^2*geom_inertia_of_circle_m1(2), \
             geom_inertia_of_circle(2));

check_nearly(geom_inertia_of_circle(0), \
             geom_inertia_of_point(0), \
             "circle point at origin");

check_equal(geom_inertia_of_circle_m1(0), \
            geom_inertia_of_point(0), \
            "circle m1 point at origin");


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_ellipse()

check_nearly(Pi*geom_inertia_of_ellipse_m1(1,1), \
             geom_inertia_of_ellipse(1,1), "ellipse 1");
check_nearly(Pi*2^2*geom_inertia_of_ellipse_m1(2,2), \
             geom_inertia_of_ellipse(2,2), "ellipse 2");

check_nearly(geom_inertia_of_ellipse_m1(0,0), \
             geom_inertia_of_point(0), \
             "ellipse point at origin");

check_equal(geom_inertia_of_ellipse_m1(1,1), \
            geom_inertia_of_circle_m1(1), \
            "ellipse equivalent circle");

check_equal(geom_inertia_of_ellipse_m1(5,0) \
            != geom_inertia_of_segment_m1(-5,5), \
            1, \
            "ellipse b=0 not equivalent to line");


\\-----------------------------------------------------------------------------
\\ geom_inertia_negate_x()

{
  check_equal(geom_inertia_negate_x(geom_inertia_of_polygon([0, 4, 4+I, I])),
              geom_inertia_of_polygon([-4, 0, I, -4+I]),
              "geom_inertia_negate_x()");
}

\\-----------------------------------------------------------------------------
\\ geom_inertia_negate_y()

{
  my(M=geom_inertia_of_polygon([0, 4, 4+I, I]),
     M_neg=geom_inertia_of_polygon([4, 0, -I, 4-I]));
  check_equal(geom_inertia_negate_y(M), M_neg,
              "geom_inertia_negate_y()");
  check_equal(matdet(M), matdet(M_neg));
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_negate_z()

{
  check_equal(geom_inertia_negate_z(geom_inertia_of_point([3,4,5])),
              geom_inertia_of_point([3,4,-5]),
              "geom_inertia_negate_z()");
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_point()

{
  my(M = geom_inertia_of_point([1,1]));   \\ first quadrant 45 degrees
  check_equal(M, [ 1, -1, 0;
                  -1,  1, 0;
                   0,  0, 2 ], "of point");
  check_equal(geom_inertia_Ix(M), 1);
  check_equal(geom_inertia_Iy(M), 1);
  check_equal(geom_inertia_Ixy(M), 1);

  check_equal(geom_inertia_I_at_pdir(M,[1,1]),0);
  check_equal(geom_inertia_I_at_pdir(M,[-1,1]),2);  \\ r = (sqrt2)^2

  my(p_max = geom_inertia_principal_xydir(M));
  p_max /= gcd(p_max);
  check_equal(p_max, [1,-1], "principal p_max");

  my(p_min = geom_point_rotate_xydir(p_max,[0,1]));
  p_min /= gcd(p_max);
  check_equal(p_min, [1,1], "principal p_min");

  \\ x=0, y=-2 is -90deg halved to -45deg which is principal max
  my(p_2max = geom_inertia_principal_2xydir(M));
  check_equal(p_2max, [0,-2], "principal p_2max");

  my(alpha_max = geom_inertia_principal_xyangle(M));
  my(alpha_min = geom_inertia_principal_xyangle(M) + Pi/2);
  check_nearly(alpha_max, -Pi/4, "point alpha_max");
  check_nearly(alpha_min,  Pi/4, "point alpha_min");

  my(I_min = geom_inertia_I_at_xyangle(M, alpha_min));
  my(I_max = geom_inertia_I_at_xyangle(M, alpha_max));
  check_nearly(I_min, 0, "I_min");
  check_nearly(I_max, 2, "I_max");  \\ distance sqrt(2)^2
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_I_at_pdir()

check_equal(geom_inertia_I_at_pdir(geom_inertia_of_point([3,4,5]),[1,0,0]), \
            4^2+5^2);
check_equal(geom_inertia_I_at_pdir(geom_inertia_of_point([3,4,5]),[0,1,0]), \
            3^2+5^2);
check_equal(geom_inertia_I_at_pdir(geom_inertia_of_point([3,4,5]),[0,0,1]), \
            3^2+4^2);


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_rectangle()

check_equal(geom_inertia_of_rectangle('w, 'h), \
            geom_inertia_of_intformal_rows(-1/2*'h, 1/2*'h,  -1/2*'w, 1/2*'w), \
            "rectangle centre from integrals")

check_equal(geom_inertia_of_rectangle(1,1), \
            [1/12, 0, 0; 0, 1/12, 0; 0, 0, 1/6]);
check_equal(matdet(geom_inertia_of_rectangle(1,1)), \
            1/864);

check_equal(geom_inertia_of_rectangle_m1(0,0), \
            geom_inertia_of_point(0), \
            "rectangle point at origin");

\\-----------------------------------------------------------------------------
\\ geom_inertia_of_rectangle_m1()

\\ h=0 same as -w/2 to +w/2
check_equal(geom_inertia_of_rectangle_m1(2,0), \
            geom_inertia_of_segment_m1(-1,1));
check_equal(geom_inertia_of_rectangle_m1('w,0), \
            geom_inertia_of_segment_m1(-'w/2,'w/2));

\\ w=0 same as -h/2 to +h/2
check_equal(geom_inertia_of_rectangle_m1(0,'h), \
            geom_inertia_of_segment_m1([0,'h/2],[0,-'h/2]));

check_equal(geom_inertia_of_rectangle_m1(0,0), \
            geom_inertia_of_point(0), \
            "rectangle m1 point at origin");


\\-----------------------------------------------------------------------------

\\ wedge across into second quadrant
{
  check_equal(geom_inertia_of_intformal_rows(0, 'h,  -'y, 'y),
              geom_inertia_of_triangle('h+'h*I, -'h+'h*I));
}

\\ wedge across into second quadrant
{
  check_equal(geom_inertia_of_intformal_rows(0, 'h,  -'y/'h*'n, 'y/'h*'p),
              geom_inertia_of_triangle('p+'h*I, -'n+'h*I));
}

check_equal(geom_inertia_negate_x(geom_inertia_of_triangle('p+'h*I, -'n+'h*I)), \
            geom_inertia_of_triangle('n+'h*I, -'p+'h*I), \
            "second quadrant formula");

check_equal(geom_inertia_of_triangle(1+I, I), \
            [1/4, -1/8, 0;  -1/8, 1/12, 0;  0, 0, 1/3]);
check_equal(geom_inertia_of_triangle(I, -1+I), \
            [1/4, 1/8, 0;  1/8, 1/12, 0;  0, 0, 1/3]);
check_equal(geom_inertia_of_triangle(-1-I, -I), \
            [1/4, -1/8, 0;  -1/8, 1/12, 0;  0, 0, 1/3]);

\\ wedge down into fourth quadrant
{
  check_equal(geom_inertia_of_intformal_rows(-'h, 0,  -'y, 'h)
              + geom_inertia_of_intformal_rows(0, 'h,  'y, 'h),
              geom_inertia_of_triangle('h - 'h*I, 'h + 'h*I));
}

\\ \\ wedge quads 1,2,3
\\ \\               +
\\ \\               |
\\ \\               |
\\ \\               |   *  1+2*I
\\ \\               |  /
\\ \\               | /
\\ \\               |/
\\ \\  -------------*---------+---
\\ \\       ____--- |
\\ \\      *        |
\\ \\               |
\\ {
\\   check_equal(geom_inertia_of_intformal_rows(0, 'h,  -'y, 'y),
\\               geom_inertia_of_triangle('h+'h*I, -'h+'h*I));
\\ }


\\-----------------------------------------------------------------------------

{
  \\  I   *
  \\     / \
  \\  0 * + *
  \\     \ /
  \\ -I   *
  my(poly = [-I,1,I,-1]);
  my(m = geom_inertia_of_polygon(poly));
  check_equal(matdet(m - geom_inertia_Ix(m)*matid(3)),
              0);  \\ X is a principal axis
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_rectangle()

{
  \\  I *--+--+--+--+--+--*
  \\    |                 |
  \\  0 |        *        |
  \\    |                 |
  \\ -I *--+--+--+--+--+-start
  \\    -3       0        3
  my(poly = [3-I,3+I, -3+I,-3-I]);
  my(m = geom_inertia_of_polygon(poly));
  check_equal(m, geom_inertia_of_rectangle(6,2));

  check_equal(geom_inertia_Ix(m), 4);   \\ smaller rotating around X axis
  check_equal(geom_inertia_Iy(m), 36);
  check_equal(geom_inertia_Ix(m) < geom_inertia_Iy(m), 1, "flat rectangle Ix smaller");
  check_equal(geom_inertia_Ixy(m), 0);  \\ symmetric
  check_nearly(geom_inertia_I_at_xyangle(m,0),    geom_inertia_Ix(m));
  check_nearly(geom_inertia_I_at_xyangle(m,Pi/2), geom_inertia_Iy(m));

  my(alpha_max = geom_inertia_principal_xyangle(m));
  my(alpha_min = geom_inertia_principal_xyangle(m) + Pi/2);
  check_nearly(alpha_max, Pi/2, "flat rectangle alpha_max");
  check_nearly(alpha_min, Pi, "flat rectangle alpha_min");  \\ smallest about X

  my(I_min = geom_inertia_I_at_xyangle(m, alpha_min));
  my(I_max = geom_inertia_I_at_xyangle(m, alpha_max));
  check_equal(I_min < I_max, 1, "I_min smaller");
}

if(option_verbose, print("tall rectangle"));
{
  \\ 2*I  *--+--*
  \\      |     |
  \\  I   |  *  |
  \\      |     |
  \\  0   |  *  |
  \\      |     |
  \\ -I   |  *  |
  \\      |     |
  \\ -2*I *--+--start
  \\     -1  0  1
  my(poly = [1-2*I,1+2*I, -1+2*I,-1-2*I]);
  my(m = geom_inertia_of_polygon(poly));
  check_equal(m, geom_inertia_of_rectangle(2,4));

  check_equal(geom_inertia_Ix(m), 32/3);   \\ smaller rotating around Y axis
  check_equal(geom_inertia_Iy(m),  8/3);
  check_equal(geom_inertia_Ix(m) > geom_inertia_Iy(m), 1, "tall rectangle Iy smaller");
  check_equal(geom_inertia_Ixy(m), 0);  \\ symmetric
  check_nearly(geom_inertia_I_at_xyangle(m,0),    geom_inertia_Ix(m));
  check_nearly(geom_inertia_I_at_xyangle(m,Pi/2), geom_inertia_Iy(m));

  my(alpha_max = geom_inertia_principal_xyangle(m));
  check_nearly(alpha_max, 0, "tall rectangle alpha_max");   \\ big about X

  my(alpha_min = alpha_max + Pi/2);
  my(I_min = geom_inertia_I_at_xyangle(m, alpha_min));
  my(I_max = geom_inertia_I_at_xyangle(m, alpha_max));
  check_equal(I_min < I_max, 1);
}

if(option_verbose, print("paralellogram"));
{
  \\                ______* I
  \\     _____------      |
  \\  0 *        *  ______* 0
  \\    |_____------      start
  \\ -I *        *
  \\    -10      0       10
  my(poly = [10, 10+I, -10, -10-I]);
  my(m = geom_inertia_of_polygon(poly));

  \\ smaller rotating around X axis
  check_equal(geom_inertia_Ix(m),   10/3, "shear rectangle Ix");
  check_equal(geom_inertia_Iy(m), 2000/3, "shear rectangle Iy");
  check_equal(geom_inertia_Ix(m) < geom_inertia_Iy(m), 1, "shear rectangle Ix smaller");
  check_equal(geom_inertia_Ixy(m), 100/3);  \\ >0 more in 1st,3rd quadrants
  check_nearly(geom_inertia_I_at_xyangle(m,0),    geom_inertia_Ix(m));
  check_nearly(geom_inertia_I_at_xyangle(m,Pi/2), geom_inertia_Iy(m));

  my(alpha_max = geom_inertia_principal_xyangle(m));
  my(alpha_min = alpha_max + Pi/2);
  check_nearly(alpha_max, -1.520713244350934062342870947,
               "alpha_max");   \\ near Y
  check_nearly(alpha_min, 0.05008308244396255688845074455); \\ near X

  my(I_min = geom_inertia_I_at_xyangle(m, alpha_min));
  my(I_max = geom_inertia_I_at_xyangle(m, alpha_max));
  check_equal(I_min < I_max, 1, "I_min smaller");
}

if(option_verbose, print("two squares"));
{
  \\  I     +--+
  \\        |  |
  \\  0  +--*--+
  \\     |  |
  \\ -I  +--start
  \\    -1  0  1
  my(poly = [-I, 0, 1, 1+I, I, 0, -1, -1-I]);
  my(m = geom_inertia_of_polygon(poly));
  check_equal(m, 2*geom_inertia_of_rectangle_xbase(1,1));

  check_equal(geom_inertia_Ix(m), 2/3);   \\ same X and Y
  check_equal(geom_inertia_Iy(m), 2/3);
  check_equal(geom_inertia_Ixy(m), 1/2);  \\ >0 more in 1st,3rd quadrants
  check_equal(geom_inertia_Ix(m) == geom_inertia_Iy(m), 1);
  check_nearly(geom_inertia_I_at_xyangle(m,0),    geom_inertia_Ix(m));
  check_nearly(geom_inertia_I_at_xyangle(m,Pi/2), geom_inertia_Iy(m));

  my(alpha_max = geom_inertia_principal_xyangle(m));
  my(alpha_min = alpha_max + Pi/2);
  check_nearly(alpha_max, -Pi/4);
  check_nearly(alpha_min, Pi/4);

  my(I_max = geom_inertia_I_at_xyangle(m, alpha_max));
  my(I_min = geom_inertia_I_at_xyangle(m, alpha_min));
  \\ print(I_min);
  \\ print(I_max);
  check_equal(I_min < I_max, 1, "I_min smaller");
}

if(option_verbose, print("big triangle"));
{
  \\         *
  \\         | \
  \\         *--*
  \\       /  \
  \\     min  max
  \\ +45deg    -45deg

  my(poly = [0, 10, 10*I]);
  my(m = geom_inertia_of_polygon(poly));

  my(alpha_max = geom_inertia_principal_xyangle(m));
  my(alpha_min = alpha_max + Pi/2);
  check_nearly(alpha_max, -Pi/4);       \\ -45deg
  check_nearly(alpha_min, Pi/4);

  my(I_min = geom_inertia_I_at_xyangle(m, alpha_min));
  my(Ix    = geom_inertia_I_at_xyangle(m, 0));
  my(Iy    = geom_inertia_I_at_xyangle(m, Pi/2));
  my(I_max = geom_inertia_I_at_xyangle(m, alpha_max));
  check_equal(I_min < I_max, 1, "big triangle I_min smaller");
  check_equal(I_min < Ix, 1);
  check_equal(I_min < Iy, 1);
  check_equal(Ix < I_max, 1);
  check_equal(Iy < I_max, 1);
  check_equal(geom_inertia_I_at_pdir(m, [1,1]),  1250/3);    \\ min
  check_equal(geom_inertia_I_at_pdir(m, [1,0]),  2*1250/3);
  check_equal(geom_inertia_I_at_pdir(m, [0,1]),  2*1250/3);
  check_equal(geom_inertia_I_at_pdir(m, [-1,1]), 3*1250/3);  \\ max -45deg
  check_equal(geom_inertia_I_at_pdir(m, [1,-1]), 3*1250/3);  \\ max -45deg

  \\           *
  \\          /|
  \\         * |   -- min 0deg
  \\          \|
  \\           *
  \\        | max 90deg

  \\ turn by -Pi/4 = -45deg
  poly *= exp(-Pi/4*I);
  m = geom_inertia_of_polygon(poly);
  check_nearly(poly, [0, (1-I)*10/sqrt(2), (1+I)*10/sqrt(2)]);

  my(p_2max = geom_inertia_principal_2xydir(m));
  check_nearly(p_2max, [-2*1250/3, 0]);
  check_equal (p_2max[1]!=0,1);
  check_nearly(p_2max[2],  0);

  my(p_max = geom_inertia_principal_xydir(m));
  \\ print(p_max);
  \\ print(geom_abs(p_2max));
  \\ print(geom_xydir_half(p_2max));
  \\ print(geom_INTERNAL_hypot(1250/3,0));
  \\ print(sign(1250/3)*1250/3);
  \\ quit;
  check_nearly(p_max[1],  0);
  check_equal (p_max[2]!=0,1);

  alpha_max = geom_inertia_principal_xyangle(m);
  alpha_min = alpha_max + Pi/2;

  check_nearly(geom_inertia_I_at_pdir(m, [1,0]),   1250/3);  \\ min X axis
  check_nearly(geom_inertia_I_at_pdir(m, [0,1]), 3*1250/3);  \\ max Y axis

  check_nearly(alpha_max, Pi/2, "big triangle alpha_max");
  check_nearly(alpha_min, Pi);

  my(rot_I_max = geom_inertia_I_at_xyangle(m, alpha_max));
  my(rot_I_min = geom_inertia_I_at_xyangle(m, alpha_min));
  check_nearly(rot_I_max, I_max, "I_max after rotation");
  check_nearly(rot_I_min, I_min, "I_min after rotation");

  check_nearly(geom_inertia_I_at_xyangle(m,0), geom_inertia_Ix(m), "Ix at alpha=0");
  check_nearly(geom_inertia_I_at_xyangle(m,Pi/2), geom_inertia_Iy(m), "Iy at alpha=Pi/2");

  check_nearly(geom_inertia_Iy(m), I_max);
  check_nearly(geom_inertia_Ix(m), I_min);
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_transpose_xy()

{
  my(R_transpose_xy = [0, 1, 0;
                       1, 0, 0;
                       0, 0, 1]);
  check_equal(matdet(R_transpose_xy), -1);  \\ not matrix of rotation

  my(M = geom_inertia_of_triangle(3, 4*I),
     T = geom_inertia_transpose_xy(M));
  check_equal(T, geom_inertia_of_triangle(4, 3*I),
              "geom_inertia_transpose_xy()");
  check_equal(T, R_transpose_xy * M * R_transpose_xy);
}


\\-----------------------------------------------------------------------------
\\ matrix rotate +90

R_plus90 = {[0, -1, 0;
             1, 0, 0;
             0, 0, 1]};
check_equal(matdet(R_plus90), 1);  \\ matrix of rotation

check_equal(R_plus90 * [1,0,0]~, [0,1,0]~);
check_equal(R_plus90 * [0,1,0]~, [-1,0,0]~);
check_equal(R_plus90^4, matid(3))

{
  my(z1 = 3,
     z2 = 4*I,
     M = geom_inertia_of_triangle(z1,z2),
     M_90 = geom_inertia_of_triangle(z1*I,z2*I));
  check_equal(R_plus90^-1 * M * R_plus90, M_90);
  check_equal(R_plus90 * M * R_plus90^-1, M_90);
}

\\-----------------------------------------------------------------------------
\\ geom_inertia_of_polygon()


\\ polygon clockwise is negative mass
{
  my(polygon = [0, 3, 4*I],
     M = geom_inertia_of_polygon(polygon),
     M_rev = geom_inertia_of_polygon(Vecrev(polygon)));
  check_equal(M, -M_rev);
}

\\-----------------------------------------------------------------------------
\\ geom_inertia_rotate_xydir()

{
  my(M = geom_inertia_of_point([33,44]));
  check_equal(geom_inertia_rotate_xydir(M,1), M, "rotate by nothing");
}
{
  my(M = geom_inertia_of_Ix_Iy_Ixy(5,6,7));
  check_equal(geom_inertia_rotate_xydir(M,-1), M,
              "rotate by 180 unchanged for plane figure");
}

{
  my(M = geom_inertia_of_point(1+I));
  check_equal(geom_inertia_rotate_xydir(M, 1+I),
              geom_inertia_of_point(geom_sqrt_as_quad(2)));
}
{
  my(M = geom_inertia_of_point(1+I));
  check_equal(geom_inertia_rotate_xydir(M, I),
              geom_inertia_of_point(1-I));
}


\\ {
\\   my(M = geom_inertia_of_point([3,4,5]));
\\   check_equal(geom_inertia_rotate_pdir(M,-1),
\\               geom_inertia_of_point([-3,-4,5]),
\\               "rotate by 180 of 3D");
\\ }
\\ {
\\   my(M = geom_inertia_of_point([3,4,5]));
\\   check_equal(geom_inertia_rotate_pdir(M, 5*I),
\\               geom_inertia_of_point([4,-3,5]),
\\               "rotate by +90 of 3D");
\\ }


{
  \\ XY rotation matrix and its adjoint
  my(n = 'x^2 + 'y^2,
     xn = 'x/n,
     yn = 'y/n);
  check_equal([ 'x, -'y, 0;
                'y,  'x, 0;
                0,  0, 1 ]
              *
              [ xn,  yn,  0;
               -yn,  xn,  0;
                0,   0,  1 ]
              ,
              matid(3));
}
test_inertia_rotate_xydir_using_matrix_multiplies(M,p) =
{
  my(x = geom_point_x(p),
     y = geom_point_y(p),
     n = simplify(x^2) + simplify(y^2),
     xn = x/n,
     yn = y/n);
  [ x,  y, 0;
    -y, x, 0;
    0,  0, 1 ]
  * M *
  [ xn, -yn,  0;
    yn,  xn,  0;
     0,   0,  1 ];
}

{
  my(M = geom_inertia_of_Ix_Iy_Ixy('Ix,'Iy,'Ixy),
     p = ['x,'y]);
  check_equal(geom_inertia_rotate_xydir(M,p),
              test_inertia_rotate_xydir_using_matrix_multiplies(M,p));
}



\\-----------------------------------------------------------------------------
\\ geom_inertia_of_intformal_rows()

{
  \\ parabola y=0 to 'h, x = 0 to 'w * ('h-y)^2
  \\
  \\ y  h|    .
  \\     |          .
  \\     |             .        x = w*(h-y)^2
  \\     |              .
  \\     |               .
  \\     |               .
  \\     +----------------------  x
  \\                     w
  check_equal(geom_inertia_of_intformal_rows(0, 'h,  0, 'w*('h-'y)^2),
              geom_inertia_of_Ix_Iy_Ixy(1/30*h^5*w,      \\ Ix
                                        1/21*h^7*w^3,    \\ Iy
                                        1/60*h^6*w^2),   \\ Ixy
              "geom_inertia_of_intformal_rows() of parabola");
  check_equal(geom_area_of_intformal_rows(0, 'h,  0, 'w*('h-'y)^2),
              1/3*h^3*w,
              "geom_area_of_intformal_rows() of parabola");

  check_equal(geom_inertia_of_intformal_rows(0, 'h,  0, 'w*('h-'y)^2)
              / geom_area_of_intformal_rows( 0, 'h,  0, 'w*('h-'y)^2),
              geom_inertia_of_Ix_Iy_Ixy(1/10*h^2,      \\ Ix
                                        1/7*h^4*w^2,   \\ Iy
                                        1/20*h^3*w),   \\ Ixy
              "geom_inertia_of_intformal_rows() of parabola");
}

{
  check_equal(geom_inertia_of_intformal_rows(0,1,  0,1, 'x),
              [1/6, -1/6, 0; -1/6, 1/4, 0; 0, 0, 5/12],
              "geom_inertia_of_intformal_rows() of linear density");

  check_equal(geom_inertia_of_intformal_rows(0,1,  0,1, 1-'x),
              [1/6, -1/12, 0; -1/12, 1/12, 0; 0, 0, 1/4],
              "geom_inertia_of_intformal_rows() of linear density");


  check_equal(geom_inertia_of_intformal_rows(0,1,  0,1),
              [1/6, -1/6, 0; -1/6, 1/4, 0; 0, 0, 5/12]
              + [1/6, -1/12, 0; -1/12, 1/12, 0; 0, 0, 1/4],
              "geom_inertia_of_intformal_rows() of linear density");
}


\\      |    a
\\    h +    * z2
\\      |   /  \
\\      |  /     \
\\      | /        \
\\      |/           \
\\      *-------------+--
\\             b      z1
\\
check_equal(geom_inertia_of_triangle('b,  'a + 'h*I), \
            geom_inertia_of_triangle_xbase('b, 'h, 'a));


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_triangle_xbase() cf quadrant part

\\      |
\\    h +
\\      | \
\\      |  \
\\      |   \
\\      *-----+--
\\            b
\\
\\ if a=0 then
\\ triangle_quadrant_Ix(b,h) = 1/12* b*h^3;
\\ triangle_quadrant_Iy(b,h) = 1/12* h*b^3;          \\ transpose
\\ triangle_quadrant_Iz(b,h) = 1/12*(h*b^3 + b*h^3); \\ Ix+Iy
\\ triangle_quadrant_Ixy(b,h) = 1/24*h^2*b^2;

{
  triangle_quadrant_m = geom_inertia_of_Ix_Iy_Ixy( 1/12 * 'b * 'h^3,
                                                  1/12 * 'b^3 * 'h,
                                                  1/24 * 'h^2 * 'b^2);
}

check_equal(geom_inertia_of_triangle_xbase('b, 'h, 0), \
            triangle_quadrant_m, \
            "triangle quadrant from xbase");

check_equal(geom_inertia_of_intformal_rows(0,'h,  0, 'b - 'y/'h*'b), \
            triangle_quadrant_m, \
            "triangle quadrant from integrals")

check_equal(geom_inertia_of_triangle('b, 'h*I), \
            triangle_quadrant_m, \
            "triangle quadrant from wedge");

{
  my(m = triangle_quadrant_m,
     b = 10, h=3);
  m = subst(m, 'b, b);
  m = subst(m, 'h, h);
  check_equal(geom_inertia_of_polygon([b, h*I, 0]),
              m,
              "triangle quadrant from polygon");
}

{
  my(m = triangle_quadrant_m,
     b = 10, h=10);
  m = subst(m, 'b, b);
  m = subst(m, 'h, h);

  my(alpha_max = geom_inertia_principal_xyangle(m));
  my(alpha_min = alpha_max + Pi/2);
  check_nearly(alpha_max, -Pi/4);
  check_nearly(alpha_min, Pi/4);

  my(I_min = geom_inertia_I_at_xyangle(m, alpha_min));
  my(I_max = geom_inertia_I_at_xyangle(m, alpha_max));
  check_equal(I_min < I_max, 1, "triangle quadrant  I_min smaller");
}


\\------------------------------------------------------------------------------
\\ geom_inertia_of_rectangle_xbase()

check_equal(geom_inertia_of_rectangle_xbase(4,1), \
            geom_inertia_of_polygon([0, 4, 4+I, I]), \
            "rectangle by polygon")

check_equal(geom_inertia_of_rectangle_xbase('w, 'h), \
            geom_inertia_of_intformal_rows(0,'h, 0,'w), \
            "rectangle from integrals")

check_equal(geom_inertia_negate_x(geom_inertia_of_rectangle_xbase('w, 'h)), \
            geom_inertia_of_intformal_rows(0,'h, -'w,0), \
            "mirror rectangle from integrals")

check_equal(matdet(geom_inertia_negate_y(geom_inertia_of_rectangle_xbase(1,1))) != 0, 1);

{
  my(m = geom_inertia_of_rectangle_xbase(1/2*'w, 1/2*'h));
  m += geom_inertia_negate_y(m);  \\ first+second quadrant
  m += geom_inertia_negate_x(m);  \\ four quadrants
  check_equal(m,
              geom_inertia_of_rectangle('w, 'h),
              "rectangle centre from four");
}

\\ umm negative w ...
\\ check_equal(geom_inertia_of_rectangle_xbase(-'w, 'h), \
\\             geom_inertia_negate_x(geom_inertia_of_rectangle_xbase('w, 'h)), \
\\             "rectangle negative");
\\
\\ matdet(geom_inertia_of_rectangle_xbase('w, 'h))
\\ matdet(geom_inertia_of_rectangle_xbase(-'w, -'h))
\\ check_equal(geom_inertia_of_rectangle_xbase(-'w, -'h), \
\\             geom_inertia_negate_y(geom_inertia_of_rectangle_xbase('w, 'h)), \
\\             "rectangle w,h both negative");


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_triangle() out of rectangle

\\      +
\\      |
\\      |   z2 =c,d
\\      |   *
\\      |  /  \
\\      | / __-* z1 =a,b
\\      |/ -
\\      *---------+---
{
  my(rect = geom_inertia_of_rectangle_xbase('a, 'd),
     bottom = geom_inertia_of_triangle_xbase('a, 'b, 'a),
     top = geom_inertia_transpose_xy(geom_inertia_of_triangle_xbase('d, 'c, 'd)),
     end = geom_inertia_of_intformal_rows('b, 'd,
                                'a - (y-'b)/('d-'b) * ('a-'c),
                                'a),
     m = rect - bottom - top - end);

  check_equal(geom_inertia_of_triangle('a+'b*I, 'c+'d*I),
              m);
}

check_equal(1/2 * (a*d - b*c), \
            geom_area_of_triangle(a+'b*I, 'c+'d*I));


\\-----------------------------------------------------------------------------
\\ test_inertia_rotate_xyplus45() -- left to the more general xydir

\\ R_plus45 is a +45 degree matrix of rotation multiplied by
\\ some sqrt2's to give integer entries.
\\     [ cos 45, - sin 45,  0 ]   [ sqrt(2),   0,     0 ]
\\     [ cos 45, - sin 45,  0 ] * [    0,   sqrt(2),  0
\\     [    0,       0,     1 ]   [    0,       0,    1 ]
\\
\\ R_minus45 similarly but -45 degree and 1/sqrt2's to give
\\ integer entries.
\\     [ cos 45, - sin 45,  0 ]   [ 1/sqrt(2),     0,      0 ]
\\     [ cos 45, - sin 45,  0 ] * [    0,      1/sqrt(2),  0 ]
\\     [    0,       0,     1 ]   [    0,          0,      1 ]
\\
R_plus45 = [ 1,-1,0; 1,1,0; 0,0,1 ];   \\ and factor sqrt2
R_minus45 = \
  mattranspose(R_plus45) * [ 1/2,0,0; 0,1/2,0; 0,0,1 ]; \\ and factor 1/sqrt2

check_equal(R_plus45 * [1;0;0], [1;1;0]);
check_equal(R_plus45 * [0;1;0], [-1;1;0]);
check_equal(R_plus45 * [-1;0;0], [-1;-1;0]);
check_equal(R_plus45 * [0;-1;0], [1;-1;0]);
check_equal(R_plus45 * R_minus45, matid(3));

test_inertia_rotate_xyplus45(M)  = R_minus45 * M * R_plus45;
test_inertia_rotate_xyminus45(M) = R_plus45  * M * R_minus45;

{
  my(m = geom_inertia_of_triangle(3, 4*I),
     m45 = test_inertia_rotate_xyplus45(m),
     m90 = test_inertia_rotate_xyplus45(m45),
     m135 = test_inertia_rotate_xyplus45(m90),
     m180 = test_inertia_rotate_xyplus45(m135));
  check_equal(matdet(m45), matdet(m));
  check_equal(m45 != m, 1);
  check_equal(m180, m, "m180");
}

{
  my(m = geom_inertia_of_triangle(3, 4*I));
  check_equal(geom_inertia_rotate_xydir(m, 1+I),
              test_inertia_rotate_xyplus45(m),
              "geom_inertia_rotate_xydir 45");
}


\\------------------------------------------------------------------------------

\\ integral between xmin,xmax = sqrt(1-y^2)
test_inertia_of_ellipse_by_intformal(a,b) =
{
  my(xf=a*Pol(sqrt((1-('y/b)^2) + O('y^200)),'y));
  geom_inertia_of_intformal_rows(-b,b, -xf,xf);
}

\\ sqrt series converges slowly, so limited accuracy
{
  check_nearly(test_inertia_of_ellipse_by_intformal(1,1),
               geom_inertia_of_ellipse(1,1),
               "geom_inertia_of_ellipse() vs test_inertia_of_ellipse_by_intformal()",
               .01);
  check_nearly(test_inertia_of_ellipse_by_intformal(5,2),
               geom_inertia_of_ellipse(5,2),
               "geom_inertia_of_ellipse() vs test_inertia_of_ellipse_by_intformal()",
               .1);
}


\\-----------------------------------------------------------------------------

\\ Don't let any a-z variables be globals.
{
  my(chars=Vecsmall("abcdefghijklmnopqrstuvwxyz"));
  check_equal(vector(#chars, ind,
                     my(ch = chars[ind],
                        str = Strchr(ch),
                        ev = eval(str));
                     if(ev==eval(Str("'",str)),1,
                        print(str," = ",ev); 0)),
              vector(#chars,i, 1));
}

\\ All variables in geom.gp not left as globals.
{
  my(test_symbols=readstr("test-symbols.txt"));
  check_equal(#test_symbols>0,1);
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     check_equal(eval(test_symbol),
                 eval(Str("'"test_symbol)),
                 Str("check no leftover global variable: "test_symbol" = "eval(test_symbol))));
}

\\-----------------------------------------------------------------------------
print("test-geom-inertia.gp end, ",check_count," tests");
quit(0)
